@@ -5,7 +5,7 @@
  *
  * Because of the extremely high rate at which log messages can be generated,
  * we need to be mindful of the performance cost of obtaining any information
- * that may be logged.	Also, it's important to keep in mind that this code may
+ * that may be logged.  Also, it's important to keep in mind that this code may
  * get called from within an aborted transaction, in which case operations
  * such as syscache lookups are unsafe.
  *
@@ -15,12 +15,12 @@
  * if we run out of memory, it's important to be able to report that fact.
  * There are a number of considerations that go into this.
  *
- * First, distinguish between re-entrant use and actual recursion.	It
+ * First, distinguish between re-entrant use and actual recursion.  It
  * is possible for an error or warning message to be emitted while the
  * parameters for an error message are being computed.	In this case
  * errstart has been called for the outer message, and some field values
- * may have already been saved, but we are not actually recursing.	We handle
- * this by providing a (small) stack of ErrorData records.	The inner message
+ * may have already been saved, but we are not actually recursing.  We handle
+ * this by providing a (small) stack of ErrorData records.  The inner message
  * can be computed and sent without disturbing the state of the outer message.
  * (If the inner message is actually an error, this isn't very interesting
  * because control won't come back to the outer message generator ... but
@@ -43,7 +43,7 @@
  * overflow.)
  *
  *
- * Portions Copyright (c) 2003-2018, PgPool Global Development Group
+ * Portions Copyright (c) 2003-2015, PgPool Global Development Group
  * Portions Copyright (c) 1996-2015, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
@@ -62,7 +62,7 @@
 #include <unistd.h>
 #include <signal.h>
 #include <ctype.h>
-#ifdef HAVE_SYSLOG
+#ifdef HAVE_VSYSLOG
 #include <syslog.h>
 #endif
 #include <string.h>
@@ -93,12 +93,15 @@ struct ONEXIT on_exit_prepare = {NULL,0L};
  * (or in the parent postmaster).
  */
 static bool atexit_callback_setup = false;
+
 /*
  * This flag is set during proc_exit() to change ereport()'s behavior,
  * so that an ereport() from an on_proc_exit routine cannot get us out
  * of the exit procedure.  We do NOT want to go back to the idle loop...
  */
+
 bool		proc_exit_inprogress = false;
+
 /* local functions */
 static void proc_exit_prepare(int code);
 
@@ -123,7 +126,7 @@ extern bool redirection_done;
  */
 emit_log_hook_type emit_log_hook = NULL;
 
-#ifdef HAVE_SYSLOG
+#ifdef HAVE_VSYSLOG
 
 /*
  * Max string length to send to syslog().  Note that this doesn't count the
@@ -270,6 +273,7 @@ errstart(int elevel, const char *filename, int lineno,
 	/* Determine whether message is enabled for server log output */
 	output_to_server = is_log_level_output(elevel, pool_config->log_min_messages);
 
+
 	/* Determine whether message is enabled for client output */
 	if (elevel != COMMERROR)
 	{
@@ -333,7 +337,7 @@ errstart(int elevel, const char *filename, int lineno,
 	if(elevel == FATAL && PG_exception_stack == NULL) /* This is startup failure. Take down main process with it */
 		edata->retcode = POOL_EXIT_FATAL;
 	else
-		edata->retcode = POOL_EXIT_NOFATAL;
+		edata->retcode = POOL_EXIT_AND_RESTART;
 	if (filename)
 	{
 		const char *slash;
@@ -571,13 +575,13 @@ int pool_error_code(const char *errcode)
 		for (;;) \
 		{ \
 			va_list		args; \
-			bool		success; \
+			int			needed; \
 			va_start(args, fmt); \
-			success = appendStringInfoVA(&buf, fmtbuf, args); \
+			needed = appendStringInfoVA(&buf, fmtbuf, args); \
 			va_end(args); \
-			if (success) \
+			if (needed == 0) \
 				break; \
-			enlargeStringInfo(&buf, buf.maxlen); \
+			enlargeStringInfo(&buf, needed); \
 		} \
 		/* Done with expanded fmt */ \
 		pfree(fmtbuf); \
@@ -614,13 +618,13 @@ int pool_error_code(const char *errcode)
 		for (;;) \
 		{ \
 			va_list		args; \
-			bool		success; \
+			int			needed; \
 			va_start(args, n); \
-			success = appendStringInfoVA(&buf, fmtbuf, args); \
+			needed = appendStringInfoVA(&buf, fmtbuf, args); \
 			va_end(args); \
-			if (success) \
+			if (needed == 0) \
 				break; \
-			enlargeStringInfo(&buf, buf.maxlen); \
+			enlargeStringInfo(&buf, needed); \
 		} \
 		/* Done with expanded fmt */ \
 		pfree(fmtbuf); \
@@ -1427,7 +1431,7 @@ GetErrorContextStack(void)
 }
 
 
-#ifdef HAVE_SYSLOG
+#ifdef HAVE_VSYSLOG
 
 /*
  * Set or update the parameters for syslog logging
@@ -1562,7 +1566,7 @@ write_syslog(int level, const char *line)
 		syslog(level, "[%lu] %s", seq, line);
 	}
 }
-#endif   /* HAVE_SYSLOG */
+#endif   /* HAVE_VSYSLOG */
 
 #ifdef WIN32
 /*
@@ -1742,15 +1746,9 @@ write_console(const char *line, int len)
 static void
 send_message_to_frontend(ErrorData *edata)
 {
-	int protoVersion;
+	int protoVersion = PROTO_MAJOR_V3; /* default protocol version is V3 also used by pcp lib */
 
-	if (pg_frontend_exists() < 0 )
-		return;
-
-	/*
-	 * Do not forward the debug messages to client before session is initialized
-	 */
-	if (edata->elevel < ERROR && pool_get_session_context(true) == NULL)
+	if (pool_frontend_exists() < 0 )
 		return;
 	/*
 	 * Leave if we are failing on sending the message to frontend.
@@ -1758,16 +1756,28 @@ send_message_to_frontend(ErrorData *edata)
 	if (++frontend_error_recursion_depth > 2)
 		return;
 
-	protoVersion = get_frontend_protocol_version();
-	set_pg_frontend_blocking(false);
+	if(processType == PT_CHILD)
+	{
+		/*
+		 * Do not forward the debug messages to client before session is initialized
+		 */
+		if (edata->elevel < ERROR && pool_get_session_context(true) == NULL)
+		{
+			frontend_error_recursion_depth--;
+			return;
+		}
+		protoVersion = get_frontend_protocol_version();
+		set_pg_frontend_blocking(false);
+	}
 
 	if (protoVersion == PROTO_MAJOR_V2)
 	{
 		char* message = edata->message?edata->message:"missing error text";
-		send_to_pg_frontend((edata->elevel < ERROR) ? "N" : "E", 1,false);
-		send_to_pg_frontend(message, strlen(message),false);
-		send_to_pg_frontend("\n", 1,true);
+		pool_send_to_frontend((edata->elevel < ERROR) ? "N" : "E", 1,false);
+		pool_send_to_frontend(message, strlen(message),false);
+		pool_send_to_frontend("\n", 1,true);
 	}
+
 	else if (protoVersion == PROTO_MAJOR_V3)
 	{
         /*
@@ -1786,11 +1796,11 @@ send_message_to_frontend(ErrorData *edata)
 		int len;
 		int thislen;
 		int sendlen;
-
+        
 		len = 0;
 		memset(data, 0, MAXDATA);
-		send_to_pg_frontend((edata->elevel < ERROR) ? "N" : "E", 1, false);
-        
+		pool_send_to_frontend((edata->elevel < ERROR) ? "N" : "E", 1, false);
+
 		/* error level */
 		thislen = snprintf(msgbuf, MAXMSGBUF, "S%s", error_severity(edata->elevel));
 		thislen = Min(thislen, MAXMSGBUF);
@@ -1846,11 +1856,14 @@ send_message_to_frontend(ErrorData *edata)
 		sendlen = len;
 		len = htonl(len + 4);
 
-		send_to_pg_frontend((char*)&len, sizeof(len), false);
-		send_to_pg_frontend(data, sendlen, true);
+		pool_send_to_frontend((char*)&len, sizeof(len), false);
+		pool_send_to_frontend(data, sendlen, true);
+
 	}
 
-	set_pg_frontend_blocking(true);
+	if (processType == PT_CHILD)
+		set_pg_frontend_blocking(true);
+
 	frontend_error_recursion_depth--;
 }
 
@@ -1969,13 +1982,7 @@ log_line_prefix(StringInfo buf, const char *line_prefix, ErrorData *edata)
 		{
 			case 'a':	/* application name */
 			{
-
-				/*
-				 * Do not use MASTER_CONNECTION macro here since it calls
-				 * pool_virtual_master_db_node_id() which eventually calls
-				 * ereport() if operated in DEBUG mode.
-				 */
-				StartupPacket *sp = session? (session->backend->slots[REAL_MASTER_NODE_ID])->sp : NULL ;
+				StartupPacket *sp = session? MASTER_CONNECTION(session->backend)->sp : NULL ;
 				const char *appname = sp? sp->application_name : "[No Connection]";
 				if (appname == NULL || *appname == '\0')
 					appname = "[unknown]";
@@ -2133,7 +2140,7 @@ send_message_to_server_log(ErrorData *edata)
 		}
 	}
 
-#ifdef HAVE_SYSLOG
+#ifdef HAVE_VSYSLOG
 	/* Write to syslog, if enabled */
     if (pool_config->logsyslog == 1)
     {
@@ -2171,7 +2178,7 @@ send_message_to_server_log(ErrorData *edata)
 		if (pool_config->logsyslog == 1)
 			write_syslog(syslog_level, buf.data);
 	}
-#endif   /* HAVE_SYSLOG */
+#endif   /* HAVE_VSYSLOG */
 
     write_console(buf.data, buf.len);
 	pfree(buf.data);
@@ -2354,6 +2361,9 @@ process_name(void)
 		case PT_PCP:
 			prefix = _("PCP CHILD");
 			break;
+		case PT_PCP_WORKER:
+			prefix = _("PCP WORKER");
+			break;
 		case PT_HB_SENDER:
 			prefix = _("WD HB SENDER");
 			break;
